/******************************************************************************
 * FILE: 
 *    spc58nn84.ld
 *
 * UC ARCHITECTURE:
 *    PowerPC
 *
 * UC FAMILY:
 *    SPC58
 *
 * DESCRIPTION:
 *    SPC58NN84 microcontroller linker file template
 *
 * History:
 *    V1.0    23.09.2017   RO   
 *       - Initial version
 *    V2.0    30.01.2018   MF
 *       - Reuse for SPC572L64 
 *    V2.1    22.02.2018   MF
 *       - adding the bss and data section for CPU0, Clear/Copy
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

/* ============================================================================
 * SYMBOLS
 * ==========================================================================*/

/* Stack size definition */
__STACK_SIZE_CORE0 = 4K;

/* Entry point for debugger in output elf file */
ENTRY(_crt0_reset)


/* ============================================================================
 * uC MEMORY REGIONS
 * ==========================================================================*/

MEMORY
{
    /* FLASH memories */
    FLASH          (rx) : org = 0x01000000, len = 0x180000 /* 1.5MB (both Flash modules as one) */
    
    /* global RAM memories */
    SRAM0          (rw)  : org = 0x40000000, len = 0x10000  /* 64 kB */
}


/* ============================================================================
 * SECTION CONTAINERS TO MEMORY REGION MAPPING
 * ==========================================================================*/

/* CRT0 header and sections */ 
REGION_ALIAS("BOOT", FLASH)

/* IVOR & ISR vectors */
REGION_ALIAS("IVOR", FLASH) 
REGION_ALIAS("ISR",  SRAM0) 

/* CORE(0) code, ram_code, const and data */
REGION_ALIAS("CODE_CPU0_",     FLASH)
REGION_ALIAS("RODATA_CPU0_",   FLASH)
REGION_ALIAS("STACK_CPU0_",    SRAM0) 
REGION_ALIAS("DATA_CPU0_",     SRAM0)
REGION_ALIAS("RAM_CODE_CPU0_", SRAM0)

/* COMMON code, const and data */
REGION_ALIAS("CODE",    FLASH)
REGION_ALIAS("RODATA",  FLASH)
REGION_ALIAS("DATA",    SRAM0)
REGION_ALIAS("RAM_CODE",SRAM0)

/* ============================================================================
 * SECTION GROUPS (INPUT -> OUTPUT)
 * ==========================================================================*/

SECTIONS
{
	/* ------------------------------------------------------------------------
     * CRT0 BOOT 
     * rchw: Boot Header searched for BAM at predefined addresses in flash
     * crt0_asm: 'C' runtime environment init
     * ----------------------------------------------------------------------*/
	
	/* RCHW Reset word */
    .rchw : FLAGS(axvl)    {
        KEEP(*(.rchw))
    } > BOOT

	/* CRT0 startup code */
    .crt0 : ALIGN(8) FLAGS(avxl)
    {
        KEEP(*(.crt0))
    } > BOOT

	/* ------------------------------------------------------------------------
     * IVOR & ISR TABLES
     * ivor_table: exception table with calls to exception handlers 
     * ivor_handlers: crt0 provided exception handlers
     * isrTable: vector table with interrupt handler entry points
     * ----------------------------------------------------------------------*/

	.ivor_table : ALIGN(256) FLAGS(avxl)
	{
        KEEP(*(.ivor_table))
	} > IVOR

	.ivor_handlers : FLAGS(avxl)
	{
        *(.ivor_handlers)
	} > IVOR

	/* ------------------------------------------------------------------------
     * CORE(x) STACK MEMORY
     * We create one Stack for each Core
     * Size is defined at the beginning of the linker file
     * ----------------------------------------------------------------------*/
	
    .CPU0.stack  : ALIGN(8) FLAGS(aw)
    {
        __SP_END_CORE0 = .;
        . +=  __STACK_SIZE_CORE0;
        __SP_INIT_CORE0 = .;
    } > STACK_CPU0_

	/* ------------------------------------------------------------------------
     * CORE(0) specific sections
	 * all that have special naming of sections starting with prefix .CPU0
     * ----------------------------------------------------------------------*/

	/* Code collection */
    .CPU0.code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .text_vle sections                                */
    } > CODE_CPU0_

	/* Code to run from IMEM */
    .CPU0.ram_code : FLAGS(axv)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .ram_code sections                                */
    } > RAM_CODE_CPU0_ AT > RODATA_CPU0_
	
	/* CONST DATA in Flash */
    .CPU0.rodata : ALIGN(8) FLAGS(arl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .rodata sections                                  */
    } > RODATA_CPU0_

	/* Init data in RAM */
    .CPU0.data : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .data sections                                    */
    } > DATA_CPU0_ AT > RODATA_CPU0_

	/* Non-init data in RAM */
	.CPU0.bss : FLAGS(awl)
    {  
        /* USER INPUT SECTION SELECTION PATTERN FOR CORE0    */
        /* .bss sections                                     */
    } > DATA_CPU0_

	/* ------------------------------------------------------------------------
     * COMMON SECTIONS
	 * all that is not assigned to specific Core in above section mapping
     * ----------------------------------------------------------------------*/

	/* Code collection */
    .code : FLAGS(axv)
    {  
        *(.text_vle*)
        *(*.text_vle*)
    } > CODE
	
	/* Code to run from SRAM */
    .ram_code : FLAGS(axv)
    {  
        *(.ramcode*)
        *(*.ramcode*)
    } > RAM_CODE AT > RODATA
	
	/* CONST DATA in Flash */
    .rodata : ALIGN(8) FLAGS(arl)
    {  
        *(.rodata*)
        *(*.rodata*)
    } > RODATA

	/* Init data in RAM */
    .data : FLAGS(awl)
    {  
        *(.data*)
        *(*.data*)
    } > DATA AT > RODATA

	/* Init data in RAM */
    .bss : FLAGS(awl)
    {  
        *(.bss*)
        *(*.bss*)
    } > DATA

	/* ------------------------------------------------------------------------
     * CLEAR & COPY TABLES
     * clear_table: data memory ranges to clear to zero
     * copy_table: data memory ranges that needs to be initialized with values
     *             init values are stored in FLASH and copied to RAM
     * ----------------------------------------------------------------------*/

	/* CORE(0) - clear data table */
	.CPU0.clear_table : ALIGN(8) FLAGS(arl)
	{
		PROVIDE(__clear_table_core0 = .);
		LONG(ADDR(.CPU0.bss))       ; LONG(SIZEOF(.CPU0.bss));
	    LONG(ADDR(.bss))            ; LONG(SIZEOF(.bss));
		LONG(-1)                    ; LONG(-1);
	} > RODATA_CPU0_

	/* CORE(0) - init data table */
    .CPU0.copy_table  : ALIGN(8) FLAGS(arl)
    {
        PROVIDE(__copy_table_core0 = .);
        LONG(LOADADDR(.CPU0.ram_code)) ; LONG(0 + ADDR(.CPU0.ram_code)) ; LONG(SIZEOF(.CPU0.ram_code));
        LONG(LOADADDR(.CPU0.data))     ; LONG(0 + ADDR(.CPU0.data))     ; LONG(SIZEOF(.CPU0.data));
        LONG(LOADADDR(.ram_code))      ; LONG(0 + ADDR(.ram_code))      ; LONG(SIZEOF(.ram_code));
        LONG(LOADADDR(.data))          ; LONG(0 + ADDR(.data))          ; LONG(SIZEOF(.data));
        LONG(-1)                       ; LONG(-1)                       ; LONG(-1);
    } > RODATA_CPU0_

	/* ------------------------------------------------------------------------
     * CRT0 CONFIG STRUCTURE
     * Initialization structure for uC Cores used in CRT0
     * all values comes from Linker file or from command line
     * ----------------------------------------------------------------------*/

	.rodata : ALIGN(8) FLAGS(arl)
	{
		PROVIDE(__crt0_config = .);    /* base of the config table */
		
		/* Core[0] */
		LONG(__SP_INIT_CORE0);         /* STACK address */ 
		LONG(SIZEOF(.CPU0.stack));     /* STACK size */
		LONG(__clear_table_core0);     /* CLEAR table */
		LONG(__copy_table_core0);      /* COPY table */
		LONG(-1);                      /* SDA pointer - short data */
		LONG(-1);                      /* SDA2 pointer - short const */
			
	} > RODATA

	/* ------------------------------------------------------------------------
     * OTHER SECTIONS
     * ----------------------------------------------------------------------*/

}


/* ============================================================================
 * OTHER SYMBOLS
 * ==========================================================================*/

/* Extended base registers (linker always searches for these symbols) */
_SDA14_BASE_ = 0x8000;
_SDA15_BASE_ = 0x8000;
_SDA16_BASE_ = 0x8000;
_SDA17_BASE_ = 0x8000;
